/*
    Copyright (C) 2007, 2009 Darrin M. Gorski

    This file is part of LCSMessengerHistory.

    LCSMessengerHistory is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    LCSMessengerHistory is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


using System;
using System.IO;
using Microsoft.Win32;
using System.Windows.Forms;
//using MessengerAPI;  // MessengerAPI seems to be broken on my machine
using CommunicatorAPI;  // I think this only works with/via Office Communicator


namespace LCSMessengerHistory {

    public class HistoryHandler {

        // the history folder (default)
        public string historyFolder = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + "\\Communicator Saved Conversations\\";

        // Messenger Interface - from CommunicatorAPI (or MessengerAPI)
        private MessengerClass Messenger;

        private MultipartyFolderPicker folderWin;

        // constructor
        public HistoryHandler() {

            // support registry override of default folder spec
            try {
                RegistryKey historyFolderKey = Registry.CurrentUser.OpenSubKey("Software\\Darrin M. Gorski\\LCSMessengerHistory");
                Object historyFolderObj = historyFolderKey.GetValue("HistoryFolder");

                if (historyFolderObj != null) {
                    historyFolder = historyFolderObj.ToString();
                }
            } catch { }

            // Init the Messenger Object
            try {
                Messenger = new MessengerClass();
            } catch (Exception ex) {
                MessageBox.Show(
                    "Unable to create messenger object.\n\n" + ex.Message,
                    "LCSMessengerHistory",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error,
                    MessageBoxDefaultButton.Button1
                );
                throw new System.ApplicationException("Can't create messenger.");
            }

            // make sure the history folder exists
            if (Directory.Exists(historyFolder) != true) {
                try {
                    Directory.CreateDirectory(historyFolder);
                } catch (Exception cde) {
                    MessageBox.Show(
                        "Unable to create history folder!\n\n" + cde.Message,
                        "LCSMessengerHistory",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error,
                        MessageBoxDefaultButton.Button1
                    );
                    throw new System.ApplicationException(
                        "Can't create history folder."
                    );
                }
            }

            Messenger.OnIMWindowDestroyed +=
                new DMessengerEvents_OnIMWindowDestroyedEventHandler(
                  this.ConversationEndedEvent
                );

            // the "choose a folder" dialog for multiparty conversations
            folderWin = new MultipartyFolderPicker(this);
        }

        // ConversationEndedEvent() - the workhorse
        private void ConversationEndedEvent(object arg) {

            IMessengerConversationWnd IMWindow = (IMessengerConversationWnd)arg;

            if (IMWindow.History == null) {
                // IMWindow.History is null if no conversation took place
                return;
            }

            IMessengerContacts contacts = (IMessengerContacts)IMWindow.Contacts;

            // pin any dialogs to the conversation window so it makes
            // sense when they get the prompt
            Win32WinWrapper msgWin = new Win32WinWrapper(IMWindow);

            DialogResult tmpRes;

            string historyFileName = "";

            if (contacts.Count != 1) { // multiparty
                tmpRes = MessageBox.Show(
                    msgWin,
                    "Save multiparty conversation?",
                    "Save conversation",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question,
                    MessageBoxDefaultButton.Button2
                );
                if (tmpRes.Equals(DialogResult.No)) {
                    return;
                }

                bool continueLoop = true;
                while (continueLoop) {
                    // have the user pick a directory for multiparty conversations
                    // also pinned to the conversation window
                    if (folderWin.ShowDialog(msgWin).Equals(DialogResult.Cancel)) {
                        return;
                    }

                    historyFileName = folderWin.getSelectedFolder();

                    if (Directory.Exists(historyFileName) != true) {
                        tmpRes = MessageBox.Show(
                            msgWin, 
                            "The selected folder does not exist.  Would you like to create it?", 
                            "LCSMessengerHistory", 
                            MessageBoxButtons.YesNo, 
                            MessageBoxIcon.Question, 
                            MessageBoxDefaultButton.Button1
                        );
                        if(tmpRes.Equals(DialogResult.Yes)) {
                            try {
                                Directory.CreateDirectory(historyFileName);
                                continueLoop = false;
                            } catch (Exception cde) {
                                MessageBox.Show(
                                    msgWin,
                                    "Unable to create folder.\n\n"
                                      + cde.Message
                                      + "\n\nPlease select another folder.",
                                    "LCSMessengerHistory", 
                                    MessageBoxButtons.OK, 
                                    MessageBoxIcon.Error, 
                                    MessageBoxDefaultButton.Button1
                                );
                            }
                        }
                    } else {
                        continueLoop = false;
                    }
                }

            } else { // single-party

                string historySubDir = ((IMessengerContact)contacts.Item(0)).SigninName;

                // make sure the history folder exists
                if (Directory.Exists(historyFolder + historySubDir) != true) {
                    try {
                        Directory.CreateDirectory(historyFolder + historySubDir);
                    } catch (Exception cde) {
                        MessageBox.Show(
                            msgWin,
                            "Unable to create history folder for contact '"
                              + historySubDir
                              + "'\n\n" + cde.Message
                              + "\n\nHistory for this conversation could not be saved.", 
                            "LCSMessengerHistory", 
                            MessageBoxButtons.OK, 
                            MessageBoxIcon.Error, 
                            MessageBoxDefaultButton.Button1
                        );
                        return;
                    }
                }

                historyFileName = historyFolder + historySubDir;

            }

            historyFileName += "\\" + DateTime.Now.ToString("dd-MMM-yyyy") + ".txt";

            StreamWriter historyFile;
            try {
                historyFile = File.AppendText(historyFileName);
            } catch (Exception fae) {
                MessageBox.Show(
                    msgWin,
                    "Unable to append/create history file.\n\n"
                      + fae.Message
                      + "\n\nHistory for this conversation could not be saved.", 
                    "LCSMessengerHistory", 
                    MessageBoxButtons.OK, 
                    MessageBoxIcon.Error, 
                    MessageBoxDefaultButton.Button1
                );
                return;
            }

            historyFile.WriteLine(
                "\r\n=================================================\r\n"
              + DateTime.Now.ToLongDateString() + " "
              + DateTime.Now.ToLongTimeString()
              + "\r\n=================================================\r\n");

            historyFile.WriteLine(IMWindow.History);
            historyFile.Close();
        }

    }
}
